@@ -1,14 +1,24 @@
+import json
 from typing import Callable, Union
 
 import requests
-
 from pymbse.commons import text_file
-from pymbse.pymbse.model_api.executor.notebook_script_executor import ScriptExecutor, NotebookExecutor
-from pymbse.pymbse.model_api.executor.notebook_converter import create_script_if_missing_or_outdated
+
+from pymbse.pymbse.config.numpy_encoder import NumpyEncoder
+from pymbse.pymbse.model_api.executor.notebook_script_executor import (
+    ScriptExecutor,
+    NotebookExecutor,
+)
+from pymbse.pymbse.model_api.executor.notebook_converter import (
+    create_script_if_missing_or_outdated,
+)
 
 from pymbse.pymbse.model_api.executor.notebook_converter import convert_ipynb_to_html
 from pymbse.pymbse.model_api.model_api import ModelAPI
-from pymbse.pymbse.model_api.snapshot.model_snapshot import ModelSnapshot, HashedModelSnapshot
+from pymbse.pymbse.model_api.snapshot.model_snapshot import (
+    ModelSnapshot,
+    HashedModelSnapshot,
+)
 
 API_ENDPOINT = "api/model_snapshots"
 
@@ -20,15 +30,22 @@ def post_model_snapshot_to_cache_decorator(execute_notebook: Callable) -> Callab
     :param execute_notebook: a function executing a notebook and returning model snapshot
     :return: a cached execute_script function that either stores the model snapshot or not
     """
+
     def wrapper(model_api: ModelAPI) -> None:
         """Wrapper function executing a notebook
 
         :param model_api: ModelAPI instance
         """
-        cached_model_snapshot = get_cached_model_snapshot(model_api.ip_with_port, model_api.root_model_metadata_hash)
+        cached_model_snapshot = get_cached_model_snapshot(
+            model_api.ip_with_port, model_api.root_model_metadata_hash
+        )
         if cached_model_snapshot is None:
             executed_model_snapshot = execute_notebook(model_api)
-            post_model_snapshot_to_cache(model_api.ip_with_port, executed_model_snapshot, model_api.root_model_metadata_hash)
+            post_model_snapshot_to_cache(
+                model_api.ip_with_port,
+                executed_model_snapshot,
+                model_api.root_model_metadata_hash,
+            )
         else:
             execute_notebook(model_api)
 
@@ -42,26 +59,35 @@ def get_model_snapshot_from_cache_decorator(execute_script: Callable) -> Callabl
     :param execute_script: an execute_script function
     :return: a cached execute_script function that either executes the function or returns a cached model snapshot
     """
+
     def wrapper(model_api: ModelAPI) -> ModelSnapshot:
-        """ Function wrapping the model_api functionality
+        """Function wrapping the model_api functionality
 
         :param model_api: ModelAPI instance
         :return: a ModelSnapshot instance either obtained from the model_api or from script execution
         """
 
-        cached_model = get_cached_model_snapshot(model_api.ip_with_port, model_api.root_model_metadata_hash)
+        cached_model = get_cached_model_snapshot(
+            model_api.ip_with_port, model_api.root_model_metadata_hash
+        )
         if cached_model is None:
-            print('Executing %s as its model snapshot is not yet cached.' %
-                  model_api.root_model_config.model_abs_path)
+            print(
+                "Executing %s as its model snapshot is not yet cached."
+                % model_api.root_model_config.model_abs_path
+            )
             cached_model = execute_script(model_api)
-            post_model_snapshot_to_cache(model_api.ip_with_port, cached_model, model_api.root_model_metadata_hash)
+            post_model_snapshot_to_cache(
+                model_api.ip_with_port, cached_model, model_api.root_model_metadata_hash
+            )
 
         return cached_model
 
     return wrapper
 
 
-def get_cached_model_snapshot(ip_with_port: str, model_snapshot_hash: str) -> Union[None, ModelSnapshot]:
+def get_cached_model_snapshot(
+    ip_with_port: str, model_snapshot_hash: str
+) -> Union[None, ModelSnapshot]:
     """Function getting a cached model from a database
 
     :param ip_with_port: a path to the database with an ip and port, e.g., localhost:8000
@@ -76,7 +102,9 @@ def get_cached_model_snapshot(ip_with_port: str, model_snapshot_hash: str) -> Un
         return HashedModelSnapshot(**response.json()).downcast_to_model_snapshot()
 
 
-def post_model_snapshot_to_cache(ip_with_port: str, model_snapshot: ModelSnapshot, model_snapshot_hash: str) -> None:
+def post_model_snapshot_to_cache(
+    ip_with_port: str, model_snapshot: ModelSnapshot, model_snapshot_hash: str
+) -> None:
     """Function posting a model snapshot with model_api to the model_api database
 
     :param ip_with_port: a path to the database with an ip and port, e.g., localhost:8000
@@ -84,10 +112,13 @@ def post_model_snapshot_to_cache(ip_with_port: str, model_snapshot: ModelSnapsho
     :param model_snapshot_hash: a model_api of the model snapshot, it is the primary key for the model_api mechanism
     """
     url = "%s/%s/" % (ip_with_port, API_ENDPOINT)
-    json_payload = model_snapshot.upcast_to_hashed_model_snapshot(model_snapshot_hash).dict()
-    response = requests.request("POST", url, json=json_payload)
+    payload_dct = model_snapshot.upcast_to_hashed_model_snapshot(
+        model_snapshot_hash
+    ).dict()
+    payload_json = json.loads(json.dumps(payload_dct, cls=NumpyEncoder))
+    response = requests.request("POST", url, json=payload_json)
     if response.status_code != 201:
-        raise ConnectionError('Could not post model snapshot to url: %s' % url)
+        raise ConnectionError("Could not post model snapshot to url: %s" % url)
 
 
 class NotebookModelAPI(ModelAPI):
@@ -98,8 +129,10 @@ class NotebookModelAPI(ModelAPI):
         :return: a string containing the HTML report content
         """
         self.execute_without_cache()
-        convert_ipynb_to_html(ipynb_abs_path=self.root_model_config.get_ipynb_out_abs_path(),
-                              html_abs_path=self.root_model_config.get_html_out_abs_path())
+        convert_ipynb_to_html(
+            ipynb_abs_path=self.root_model_config.get_ipynb_out_abs_path(),
+            html_abs_path=self.root_model_config.get_html_out_abs_path(),
+        )
         return text_file.read(self.root_model_config.get_html_out_abs_path())
 
     def get_ipynb_report(self) -> str:
@@ -113,30 +146,37 @@ class NotebookModelAPI(ModelAPI):
 
     @post_model_snapshot_to_cache_decorator
     def execute_without_cache(self) -> ModelSnapshot:
-        return NotebookExecutor(model_dependency_config=self.root_model_config,
-                                input_parameters=self.input_parameters).execute_with_snapshot()
+        return NotebookExecutor(
+            model_dependency_config=self.root_model_config,
+            input_parameters=self.input_parameters,
+        ).execute_with_snapshot()
 
     @get_model_snapshot_from_cache_decorator
     def execute(self) -> ModelSnapshot:
-        return NotebookExecutor(model_dependency_config=self.root_model_config,
-                                input_parameters=self.input_parameters).execute_with_snapshot()
+        return NotebookExecutor(
+            model_dependency_config=self.root_model_config,
+            input_parameters=self.input_parameters,
+        ).execute_with_snapshot()
 
 
 class ScriptModelAPI(ModelAPI):
-
     @get_model_snapshot_from_cache_decorator
     def execute(self) -> ModelSnapshot:
-        return ScriptExecutor(model_dependency_config=self.root_model_config,
-                              input_parameters=self.input_parameters
-                              ).execute_with_snapshot()
+        return ScriptExecutor(
+            model_dependency_config=self.root_model_config,
+            input_parameters=self.input_parameters,
+        ).execute_with_snapshot()
 
 
-def create_script_if_missing_or_outdated_decorator(script_dependent_function: Callable) -> Callable:
+def create_script_if_missing_or_outdated_decorator(
+    script_dependent_function: Callable,
+) -> Callable:
     """
 
     :param script_dependent_function: a function executing
     :return: a script_dependent_function with ensured script presence
     """
+
     def wrapper(self: ModelAPI, *args, **kwargs) -> Callable:
         """Wrapper function executing the script_dependent_function
 
@@ -148,15 +188,17 @@ def create_script_if_missing_or_outdated_decorator(script_dependent_function: Ca
         notebook_abs_path = self.root_model_config.model_abs_path
         script_abs_path = self.root_model_config.get_script_abs_path()
         model_name_without_ext = self.root_model_config.get_model_name_without_ext()
-        create_script_if_missing_or_outdated(notebook_abs_path=notebook_abs_path,
-                                             script_abs_path=script_abs_path,
-                                             model_name_without_ext=model_name_without_ext)
+        create_script_if_missing_or_outdated(
+            notebook_abs_path=notebook_abs_path,
+            script_abs_path=script_abs_path,
+            model_name_without_ext=model_name_without_ext,
+        )
         return script_dependent_function(self, *args, **kwargs)
+
     return wrapper
 
 
 class NotebookScriptModelAPI(ScriptModelAPI, NotebookModelAPI):
-
     @create_script_if_missing_or_outdated_decorator
     def get_figures_of_merit(self) -> dict:
         return super().get_figures_of_merit()
